(* Parse boolean expression *)

open Question4

let rec pretty_print = function
    Bool b -> Format.printf "%b" b
  | Var  x -> Format.printf "%s" x
  | Not  e -> Format.printf "@[<h>!("; pretty_print e; Format.printf ")@]"
  | Or (e1,e2) ->
      begin
	Format.printf "@[<h>(";
	pretty_print e1;
	Format.printf " || ";
	pretty_print e2;
	Format.printf "@])"
      end
  | And (e1,e2) ->
      begin
	Format.printf "@[<h>(";
	pretty_print e1;
	Format.printf " && ";
	pretty_print e2;
	Format.printf "@])"
      end
  | Impl (e1,e2) ->
      begin
	Format.printf "@[<h>(";
	pretty_print e1;
	Format.printf " => ";
	pretty_print e2;
	Format.printf "@])"
      end
  | Eq (e1,e2) ->
      begin
	Format.printf "@[<h>(";
	pretty_print e1;
	Format.printf " <=> ";
	pretty_print e2;
	Format.printf "@])"
      end

module StringSet = Set.Make(String)

let rec getVars = function
    Bool _ -> StringSet.empty
  | Var x -> StringSet.singleton x
  | Not e -> getVars e
  | Or(e1,e2) | And(e1,e2) | Impl(e1,e2) | Eq(e1,e2) ->
      StringSet.union (getVars e1) (getVars e2)

let parseExpr file =
  let lexbuf = Lexing.from_channel (open_in file)
  in
    Parser.main Lexer.token lexbuf


